--- rc.M.orig	2007-07-10 01:58:25.000000000 +0200
+++ rc.M	2007-07-10 01:59:53.000000000 +0200
@@ -30,6 +30,19 @@
 # Save the contents of 'dmesg':
 /bin/dmesg -s 65536 > /var/log/dmesg
 
+# If we are returning from single-user mode we will need to restart
+# udevd on systems that use udev:
+if ! ps axc | grep -wq udevd ; then
+  if grep -wq sysfs /proc/mounts && grep -wq tmpfs /proc/filesystems ; then
+    if ! grep -wq nohotplug /proc/cmdline ; then
+      if [ -x /etc/rc.d/rc.udev ]; then
+        /bin/sh /etc/rc.d/rc.udev start
+        /bin/sh /lib/udev/udev.count_events.sh
+      fi
+    fi
+  fi
+fi
+
 # Start the system logger.
 if [ -x /etc/rc.d/rc.syslog -a -x /usr/sbin/syslogd -a -d /var/log ]; then
   . /etc/rc.d/rc.syslog start
@@ -60,11 +73,25 @@
   . /etc/rc.d/rc.inet1
 fi
 
-# Initialize the hotplugging subsystem for Cardbus, IEEE1394, PCI, and USB devices:
-if [ -x /etc/rc.d/rc.hotplug -a -r /proc/modules ]; then
-  # Don't run hotplug if 'nohotplug' was given at boot.
-  if ! grep nohotplug /proc/cmdline 1> /dev/null 2> /dev/null ; then
-    echo "Activating hardware detection:  /etc/rc.d/rc.hotplug start"
+# Initialize the hotplugging subsystem for Cardbus, IEEE1394, PCI, and
+# USB devices.  On a 2.4.x kernel this will run the old hotplug system
+# started with /etc/rc.d/rc.hotplug.  On a newer kernel, hotplugging
+# will be handled by udev.  If you run a 2.6+ kernel, don't shut off
+# udev unless you also don't want hotplugging support.
+if cat /proc/mounts | grep -wq sysfs ; then
+  if ! grep -wq nohotplug /proc/cmdline ; then
+    if [ -x /etc/rc.d/rc.udev ]; then
+      if [ -d /dev/.udev/failed ]; then
+       for i in /dev/.udev/failed/*; do
+         echo "add" > "${i}/uevent"
+       done
+      fi
+    elif [ -x /etc/rc.d/rc.hotplug ]; then
+      . /etc/rc.d/rc.hotplug start
+    fi
+  fi
+elif [ -x /etc/rc.d/rc.hotplug -a -w /proc/sys/kernel/hotplug ]; then
+  if ! grep -wq nohotplug /proc/cmdline ; then
     . /etc/rc.d/rc.hotplug start
   fi
 fi
@@ -126,6 +153,16 @@
 #  /usr/sbin/smartd
 #fi
 
+# If we're using udev, make /dev/cdrom and any other optical drive symlinks
+# if some udev rule hasn't made them already:
+if cat /proc/mounts | grep -wq sysfs && cat /proc/filesystems | grep -wq tmpfs ; then
+  if ! grep -wq nohotplug /proc/cmdline ; then
+    if [ -x /lib/udev/rc.optical-symlinks -a -x /etc/rc.d/rc.udev ]; then
+      /bin/sh /lib/udev/rc.optical-symlinks
+    fi
+  fi
+fi
+
 # Monitor the UPS with genpowerd.
 # To use this, uncomment this section and edit your settings in
 # /etc/genpowerd.conf (serial device, UPS type, etc).  For more information,
